# -*- coding: utf-8 -*-

from floppyforms.widgets import ClearableFileInput

class ImageEdit(ClearableFileInput):
    template_name = 'coop_cms/widgets/imageedit.html'
    
    def __init__(self, update_url, thumbnail_src, *args, **kwargs):
        super(ImageEdit, self).__init__(*args, **kwargs)
        self._extra_context = {
            'update_url': update_url,
            'thumbnail_src': thumbnail_src
        }
        
    def get_context(self, *args, **kwargs):
        context = super(ImageEdit, self).get_context(*args, **kwargs)
        context.update(self._extra_context)
        return context


# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Removing unique constraint on 'NewsletterItem', fields ['content_type', 'object_id']
        db.delete_unique('coop_cms_newsletteritem', ['content_type_id', 'object_id'])

        # Deleting model 'Newsletter'
        db.delete_table('coop_cms_newsletter')

        # Removing M2M table for field items on 'Newsletter'
        db.delete_table('coop_cms_newsletter_items')

        # Deleting model 'NewsletterItem'
        db.delete_table('coop_cms_newsletteritem')

        # Deleting model 'NewsletterSending'
        db.delete_table('coop_cms_newslettersending')


    def backwards(self, orm):
        # Adding model 'Newsletter'
        db.create_table('coop_cms_newsletter', (
            ('content', self.gf('django.db.models.fields.TextField')(default='<br>', blank=True)),
            ('template', self.gf('django.db.models.fields.CharField')(default='', max_length=200, blank=True)),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('subject', self.gf('django.db.models.fields.CharField')(default='', max_length=200, blank=True)),
        ))
        db.send_create_signal('coop_cms', ['Newsletter'])

        # Adding M2M table for field items on 'Newsletter'
        db.create_table('coop_cms_newsletter_items', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('newsletter', models.ForeignKey(orm['coop_cms.newsletter'], null=False)),
            ('newsletteritem', models.ForeignKey(orm['coop_cms.newsletteritem'], null=False))
        ))
        db.create_unique('coop_cms_newsletter_items', ['newsletter_id', 'newsletteritem_id'])

        # Adding model 'NewsletterItem'
        db.create_table('coop_cms_newsletteritem', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('content_type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['contenttypes.ContentType'])),
            ('object_id', self.gf('django.db.models.fields.PositiveIntegerField')()),
        ))
        db.send_create_signal('coop_cms', ['NewsletterItem'])

        # Adding unique constraint on 'NewsletterItem', fields ['content_type', 'object_id']
        db.create_unique('coop_cms_newsletteritem', ['content_type_id', 'object_id'])

        # Adding model 'NewsletterSending'
        db.create_table('coop_cms_newslettersending', (
            ('scheduling_dt', self.gf('django.db.models.fields.DateTimeField')(default=None, null=True, blank=True)),
            ('newsletter', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['coop_cms.Newsletter'])),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('sending_dt', self.gf('django.db.models.fields.DateTimeField')(default=None, null=True, blank=True)),
        ))
        db.send_create_signal('coop_cms', ['NewsletterSending'])


    models = {
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'coop_cms.articlecategory': {
            'Meta': {'object_name': 'ArticleCategory'},
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'logo': ('sorl.thumbnail.fields.ImageField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'ordering': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'slug': ('django_extensions.db.fields.AutoSlugField', [], {'allow_duplicates': 'False', 'max_length': '100', 'separator': "u'-'", 'blank': 'True', 'unique': 'True', 'populate_from': "'name'", 'overwrite': 'False'})
        },
        'coop_cms.document': {
            'Meta': {'object_name': 'Document'},
            'created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'blank': 'True'}),
            'file': ('django.db.models.fields.files.FileField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_private': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '200', 'blank': 'True'})
        },
        'coop_cms.image': {
            'Meta': {'object_name': 'Image'},
            'created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'blank': 'True'}),
            'file': ('django.db.models.fields.files.ImageField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '200', 'blank': 'True'})
        },
        'coop_cms.link': {
            'Meta': {'object_name': 'Link'},
            'created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'blank': 'True'}),
            'url': ('django.db.models.fields.CharField', [], {'max_length': '200'})
        },
        'coop_cms.navnode': {
            'Meta': {'object_name': 'NavNode'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'in_navigation': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'ordering': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'default': '0', 'to': "orm['coop_cms.NavNode']", 'null': 'True', 'blank': 'True'}),
            'tree': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['coop_local.NavTree']"})
        },
        'coop_cms.navtype': {
            'Meta': {'object_name': 'NavType'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']", 'unique': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label_rule': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'search_field': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '200', 'blank': 'True'})
        },
        'coop_cms.pieceofhtml': {
            'Meta': {'object_name': 'PieceOfHtml'},
            'content': ('django.db.models.fields.TextField', [], {'default': "''", 'blank': 'True'}),
            'div_id': ('django.db.models.fields.CharField', [], {'max_length': '100', 'db_index': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        },
        'coop_local.link': {
            'Meta': {'object_name': 'Link'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']", 'null': 'True', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'null': 'True', 'blank': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'object_label': ('django.db.models.fields.CharField', [], {'max_length': '150', 'null': 'True', 'blank': 'True'}),
            'object_uri': ('django.db.models.fields.URLField', [], {'default': "'http://'", 'max_length': '200', 'blank': 'True'}),
            'predicate': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['coop_local.LinkProperty']"})
        },
        'coop_local.linkproperty': {
            'Meta': {'object_name': 'LinkProperty'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'uri': ('django.db.models.fields.URLField', [], {'max_length': '200', 'blank': 'True'})
        },
        'coop_local.navtree': {
            'Meta': {'object_name': 'NavTree'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_update': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'default': "'default'", 'unique': 'True', 'max_length': '100', 'db_index': 'True'}),
            'sites': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['sites.Site']", 'symmetrical': 'False'}),
            'types': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['coop_cms.NavType']", 'symmetrical': 'False', 'blank': 'True'}),
            'uri': ('django.db.models.fields.CharField', [], {'max_length': '250', 'null': 'True', 'blank': 'True'}),
            'uri_mode': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '1'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "'kmbNr7Vv4XqCscGkKTDK2J'", 'max_length': '50', 'null': 'True'})
        },
        'sites.site': {
            'Meta': {'ordering': "('domain',)", 'object_name': 'Site', 'db_table': "'django_site'"},
            'domain': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        }
    }

    complete_apps = ['coop_cms']


# -*- coding: utf-8 -*-

# from django.core.management.base import BaseCommand
# from datetime import datetime
# from coop_cms.utils import send_newsletter
# from coop_cms.models import NewsletterSending

# class Command(BaseCommand):
#     help = u"send newsletter"

#     def handle(self, *args, **options):
#         #look for emailing to be sent
#         verbose = options.get('verbosity', 1)
        
#         if len(args)>0:
#             dests = args[0].split(";")
#         else:
#             print 'usage: python manage.py send_newsletter toto@toto.fr;titi@titi.fr'
        
#         sendings = NewsletterSending.objects.filter(scheduling_dt__lte=datetime.now(), sending_dt=None)
#         for sending in sendings:
#             if verbose:
#                 print 'send_newsletter {1} to {0} addresses'.format(len(dests), sending.newsletter)
            
#             nb_sent = send_newsletter(sending.newsletter, dests)
            
#             if verbose:
#                 print nb_sent, "emails sent"
            
#             sending.sending_dt = datetime.now()
#             sending.save()
#         


# -*- coding: utf-8 -*-
from django.db import models
from coop_cms.models import BaseArticle
from django.contrib.auth.models import User

class Article(BaseArticle):
    author = models.ForeignKey(User, blank=True, default=None, null=True)

class ModeratedArticle(Article):
    class Meta:
        proxy = True
        
    def can_publish_article(self, user):
        return user.is_superuser
    
class PrivateArticle(Article):
    class Meta:
        proxy = True
        
    def can_view_article(self, user):
        return (self.author == user)
        
    def can_edit_article(self, user):
        return (self.author == user)
        
    def can_publish_article(self, user):
        return (self.author == user)


# -*- coding: utf-8 -*-
from django.contrib import admin
from django.core.urlresolvers import reverse
import models
from forms import NavTypeForm, ArticleAdminForm  # , NewsletterItemAdminForm, NewsletterAdminForm
from django.contrib.contenttypes.models import ContentType
from django.utils.translation import ugettext_lazy as _
from coop_cms.settings import get_article_class, get_navTree_class
from sorl.thumbnail.admin import AdminImageMixin


class NavNodeAdmin(admin.ModelAdmin):
    list_display = ["label", 'parent', 'ordering', 'in_navigation', 'content_type', 'object_id']

admin.site.register(models.NavNode, NavNodeAdmin)


class NavTypeAdmin(admin.ModelAdmin):
    form = NavTypeForm

admin.site.register(models.NavType, NavTypeAdmin)


class NavTreeAdmin(admin.ModelAdmin):
    list_display = ['__unicode__', 'name', 'navtypes_list']
    list_editable = ['name']
    list_filters = ['id']

    def nodes_li(self, tree):
        root_nodes = tree.get_root_nodes()
        nodes_li = u''.join([node.as_jstree() for node in root_nodes])
        return nodes_li

    def navtypes_list(self, tree):
        if tree.types.count() == 0:
            return _(u'All')
        else:
            return u' - '.join([unicode(x) for x in tree.types.all()])
    navtypes_list.short_description = _(u'navigable types')

    def change_view(self, request, object_id, extra_context=None):
        extra_context = extra_context or {}
        tree = models.get_navTree_class().objects.get(id=object_id)
        extra_context['navtree'] = tree
        extra_context['navtree_nodes'] = self.nodes_li(tree)
        return super(NavTreeAdmin, self).change_view(request, object_id,
            extra_context=extra_context)

admin.site.register(get_navTree_class(), NavTreeAdmin)


class ArticleAdmin(admin.ModelAdmin):
    form = ArticleAdminForm
    list_display = ['slug', 'title', 'publication', 'is_homepage', 'in_newsletter', 'category', 'modified']
    list_editable = ['publication', 'is_homepage', 'in_newsletter', 'category']
    readonly_fields = ['slug', 'created', 'modified']
    fieldsets = (
        #(_('Navigation'), {'fields': ('navigation_parent',)}),
        (_('General'), {'fields': ('slug', 'title', 'content')}),
        (_('Advanced'), {'fields': ('template', 'category', 'logo', 'is_homepage', 'in_newsletter')}),
        (_('Publication'), {'fields': ('publication', 'created', 'modified')}),
        (_('Summary'), {'fields': ('summary',)}),
        (_('Debug'), {'fields': ('temp_logo',)}),
    )

    def get_form(self, request, obj=None, **kwargs):
        form = super(ArticleAdmin, self).get_form(request, obj, **kwargs)
        form.current_user = request.user
        return form
admin.site.register(get_article_class(), ArticleAdmin)

admin.site.register(models.Link)
admin.site.register(models.Document)
admin.site.register(models.Image)
admin.site.register(models.PieceOfHtml)
# admin.site.register(models.NewsletterSending)


class ArticleCategoryAdmin(admin.ModelAdmin, AdminImageMixin):
    list_display = ['logo_list_display','name', 'ordering']
    list_display_links = ('logo_list_display', 'name')
    list_editable = ['ordering']
admin.site.register(models.ArticleCategory, ArticleCategoryAdmin)

#class NewsletterItemAdmin(admin.ModelAdmin):
#    form = NewsletterItemAdminForm
#    list_display = ['content_object']
#    fieldsets = (
#        (_('Article'), {'fields': ('object_id', 'content_type')}),
#    )
#
#admin.site.register(models.NewsletterItem, NewsletterItemAdmin)


# class NewsletterAdmin(admin.ModelAdmin):
#     form = NewsletterAdminForm

#     def get_form(self, request, obj=None, **kwargs):
#         form = super(NewsletterAdmin, self).get_form(request, obj, **kwargs)
#         form.current_user = request.user
#         return form

# admin.site.register(models.Newsletter, NewsletterAdmin)



# # -*- coding: utf-8 -*-

# from django.core.management.base import BaseCommand
# from datetime import datetime
# from coop_cms.settings import get_newsletter_item_classes
# from coop_cms.models import create_newsletter_item

# class Command(BaseCommand):
#     help = u"force the creation of every newsletter items"

#     def handle(self, *args, **options):
#         #look for emailing to be sent
#         verbose = options.get('verbosity', 1)
        
#         for klass in get_newsletter_item_classes():
#             for instance in klass.objects.all():
#                 item, status = create_newsletter_item(instance)
#                 if item and status and verbose:
#                     print 'create item for', klass.__name__, '-', instance.id, ':', instance
#                 else:
#                     if status and verbose:
#                         print 'delete item for', klass.__name__, '-', instance.id, ':', instance
#         if verbose:
#             print "done"        


# -*- coding: utf-8 -*-

from HTMLParser import HTMLParser
from re import sub
from sys import stderr
from traceback import print_exc
from django.core.mail import get_connection, EmailMultiAlternatives
from coop_cms.html2text import html2text
from django.conf import settings
from django.template.loader import get_template
from django.template import Context

class _DeHTMLParser(HTMLParser):
    def __init__(self):
        HTMLParser.__init__(self)
        self.__text = []

    def handle_data(self, data):
        text = data.strip()
        if len(text) > 0:
            text = sub('[ \t\r\n]+', ' ', text)
            self.__text.append(text + ' ')

    def handle_starttag(self, tag, attrs):
        if tag == 'p':
            self.__text.append('\n\n')
        elif tag == 'br':
            self.__text.append('\n')

    def handle_startendtag(self, tag, attrs):
        if tag == 'br':
            self.__text.append('\n\n')

    def text(self):
        return ''.join(self.__text).strip()


# copied from http://stackoverflow.com/a/3987802/117092
def dehtml(text):
    try:
        parser = _DeHTMLParser()
        parser.feed(text)
        parser.close()
        return parser.text()
    except:
        print_exc(file=stderr)
        return text
    
def make_links_absolute(html_content):
    """replace all local url with absolute url"""
    import re
    #regex = """<.*(?P<tag>href|src)\s*=\s*["'](?P<url>.+?)["'].*>"""
    #regex = """<.*href|src\s*=\s*["'](?P<url>.+?)["'].*>"""
    
    def make_abs(match):
        #Thank you : http://www.gawel.org/howtos/python-re-sub
        start = match.group('start')
        url = match.group('url')
        if url.startswith('..'):
            url = url[2:]
        while url.startswith('/..'):
            url = url[3:]
        if url.startswith('/'):
            url = '%s%s' % (settings.COOP_CMS_SITE_PREFIX, url)
        end = match.group('end')
        return start + url + end
    
    a_pattern = re.compile(r'(?P<start>.*?href=")(?P<url>\S+)(?P<end>".*?)')
    html_content = a_pattern.sub(make_abs, html_content)
    
    img_pattern = re.compile(r'(?P<start>.*?src=")(?P<url>\S+)(?P<end>".*?)')
    html_content = img_pattern.sub(make_abs, html_content)

    return html_content
    
# def send_newsletter(newsletter, dests):

#     t = get_template(newsletter.get_template_name())
#     context_dict = {
#         'title': newsletter.subject, 'newsletter': newsletter, 'by_email': True,
#         'SITE_PREFIX': settings.COOP_CMS_SITE_PREFIX,
#         'MEDIA_URL': settings.MEDIA_URL, 'STATIC_URL': settings.STATIC_URL,
#     }
#     html_text = t.render(Context(context_dict))
#     html_text = make_links_absolute(html_text)

#     emails = []
#     connection = get_connection()
#     from_email = settings.COOP_CMS_FROM_EMAIL
#     reply_to = getattr(settings, 'COOP_CMS_REPLY_TO', None)
#     headers = {'Reply-To': reply_to} if reply_to else None

#     for addr in dests:
#         text = html2text(html_text)
#         email = EmailMultiAlternatives(newsletter.subject, text, from_email, [addr], headers=headers)
#         email.attach_alternative(html_text, "text/html")
#         emails.append(email)
#     return connection.send_messages(emails)


from django import forms
from coop_cms.models import NavType, NavNode # Newsletter, NewsletterSending
from django.contrib.contenttypes.models import ContentType
from settings import get_navigable_content_types
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext as _
from djaloha.widgets import AlohaInput
import floppyforms
import re
from django.conf import settings
from coop_cms.settings import get_article_class, get_article_templates, get_navTree_class  #,get_newsletter_templates
from coop_cms.widgets import ImageEdit
from django.core.urlresolvers import reverse
from coop_cms.utils import dehtml
from datetime import datetime
try:
    from chosen.widgets import ChosenSelectMultiple
except ImportError:
    print "chosen missing"
    pass

class NavTypeForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super(NavTypeForm, self).__init__(*args, **kwargs)
        self.fields['content_type'].widget = forms.Select(choices=get_navigable_content_types())

    def clean_label_rule(self):
        rule = self.cleaned_data['label_rule']
        if rule == NavType.LABEL_USE_GET_LABEL:
            ct = self.cleaned_data['content_type']
            if not 'get_label' in dir(ct.model_class()):
                raise ValidationError(_(u"Invalid rule for this content type: The object class doesn't have a get_label method"))
        return rule

    class Meta:
        model = NavType

class ArticleForm(floppyforms.ModelForm):

    def __init__(self, *args, **kwargs):
        super(ArticleForm, self).__init__(*args, **kwargs)
        self.article = kwargs.get('instance', None)
        self.set_logo_size()

    class Meta:
        model = get_article_class()
        fields = ('title', 'content', 'logo')
        widgets = {
            'title': AlohaInput(text_color_plugin=False),
            'content': AlohaInput(text_color_plugin=False),
        }

    def set_logo_size(self, logo_size=None):
        thumbnail_src = self.logo_thumbnail(logo_size)
        update_url = reverse('coop_cms_update_logo', args=[self.article.id])
        self.fields['logo'].widget = ImageEdit(update_url, thumbnail_src.url if thumbnail_src else '')

    def logo_thumbnail(self, logo_size=None):
        if self.article:
            return self.article.logo_thumbnail(True, logo_size=logo_size)

    # class Media:
    #     css = {
    #         'all': ('css/colorbox.css',),
    #     }
    #     js = ('js/jquery.form.js', 'js/jquery.pageslide.js', 'js/jquery.colorbox-min.js', 'js/colorbox.coop.js')


    def clean_title(self):
        title = self.cleaned_data['title'].strip()
        if title[-4:].lower() == '<br>':
            title = title[:-4]
        if not title:
            raise ValidationError(_(u"Title can not be empty"))

        #if re.search(u'<(.*)>', title):
        #    raise ValidationError(_(u'HTML content is not allowed in the title'))

        return title


def get_node_choices():
    prefix = "--"
    #choices = [(None, _(u'<not in navigation>')), (0, _(u'<root node>'))]
    #for root_node in NavNode.objects.filter(parent__isnull=True).order_by('ordering'):
    #    for (progeny, level) in root_node.get_progeny():
    #        choices.append((progeny.id, prefix*level+progeny.label))
    #return choices
    choices = [(None, _(u'<not in navigation>'))]
    for tree in get_navTree_class().objects.all():
        choices.append((-tree.id, tree.name))
        for root_node in NavNode.objects.filter(tree=tree, parent__isnull=True).order_by('ordering'):
            for (progeny, level) in root_node.get_progeny():
                choices.append((progeny.id, prefix*(level+1)+progeny.label))
    return choices

def get_navigation_parent_help_text():
    return get_article_class().navigation_parent.__doc__

# class NewsletterItemAdminForm(forms.ModelForm):
#     def __init__(self, *args, **kwargs):
#         super(NewsletterItemAdminForm, self).__init__(*args, **kwargs)
#         self.item = kwargs.get('instance', None)
#         article_choices = [(a.id, unicode(a)) for a in get_article_class().objects.all()]
#         self.fields['object_id'] = forms.ChoiceField(
#            choices=article_choices, required=True, help_text=_(u"Select an article")
#         )
#         self.fields['content_type'].required = False
#         self.fields['content_type'].widget = forms.HiddenInput()

#     def clean_content_type(self):
#         return ContentType.objects.get_for_model(get_article_class())


class ArticleFormWithNavigation(forms.ModelForm):
    navigation_parent = forms.ChoiceField()

    def __init__(self, *args, **kwargs):
        super(ArticleFormWithNavigation, self).__init__(*args, **kwargs)
        self.article = kwargs.get('instance', None)
        self.fields['navigation_parent'] = forms.ChoiceField(
            choices=get_node_choices(), required=False, help_text=get_navigation_parent_help_text()
        )

    def clean_navigation_parent(self):
        parent_id = self.cleaned_data['navigation_parent']
        parent_id = int(parent_id) if parent_id != 'None' else None
        #if self.article:
        #    ct = ContentType.objects.get_for_model(get_article_class())
        #    try:
        #        node = NavNode.objects.get(object_id=self.article.id, content_type=ct)
        #        #raise ValidationError if own parent or child of its own child
        #        node.check_new_navigation_parent(parent_id)
        #    except NavNode.DoesNotExist:
        #        pass
        return parent_id

    def save(self, commit=True):
        article = super(ArticleFormWithNavigation, self).save(commit=False)
        parent_id = self.cleaned_data['navigation_parent']
        if article.id:
            article.navigation_parent = parent_id
        else:
            setattr(article, '_navigation_parent', parent_id)
        if commit:
            article.save()
        return article


class ArticleAdminForm(forms.ModelForm):

    def __init__(self, *args, **kwargs):
        super(ArticleAdminForm, self).__init__(*args, **kwargs)
        self.article = kwargs.get('instance', None)
        templates = get_article_templates(self.article, self.current_user)
        if templates:
            self.fields['template'].widget = forms.Select(choices=templates)

    class Meta:
        model = get_article_class()
        widgets = {
            'title': forms.TextInput(attrs={'size': 100})
        }


class AddImageForm(forms.Form):
    image = forms.ImageField(required=True, label = _('Image'),)
    descr = forms.CharField(required=False, widget=forms.TextInput(
        attrs={'size': '35', 'placeholder': _(u'Optional description'),}),
        label = _('Description'),
    )


class AddDocForm(forms.Form):
    doc = forms.FileField(required=True, label = _('File'),)
    descr = forms.CharField(required=False, widget=forms.TextInput(
        attrs={'size': '35', 'placeholder': _(u'Optional description'),}),
        label = _('Description'),
    )
    is_private = forms.BooleanField(required=False)


class ArticleTemplateForm(forms.Form):
    def __init__(self, article, user, *args, **kwargs):
        super(ArticleTemplateForm, self).__init__(*args, **kwargs)
        choices = get_article_templates(article, user)
        if choices:
            self.fields["template"] = forms.ChoiceField(choices=choices)
        else:
            self.fields["template"] = forms.CharField()
        self.fields["template"].initial = article.template


class ArticleLogoForm(forms.Form):
    image = forms.ImageField(required=True, label = _('Logo'),)


class ArticleSettingsForm(forms.ModelForm):
    class Meta:
        model = get_article_class()
        fields = ('template', 'category', 'in_newsletter', 'summary')

    def __init__(self, user, *args, **kwargs):
        article = kwargs['instance']

        try:
            initials = kwargs['initial']
        except:
            initials = {}
        summary = article.summary
        if not summary:
            summary = dehtml(article.content)[:250]
        initials.update({'summary': summary})
        kwargs['initial'] = initials
        super(ArticleSettingsForm, self).__init__(*args, **kwargs)

        choices = get_article_templates(article, user)
        if choices:
            self.fields["template"] = forms.ChoiceField(choices=choices)
        else:
            self.fields["template"] = forms.CharField()


class NewArticleForm(ArticleFormWithNavigation):
    class Meta:
        model = get_article_class()
        fields = ('title', 'template', 'publication')

    def __init__(self, user, *args, **kwargs):
        super(NewArticleForm, self).__init__(*args, **kwargs)
        choices = get_article_templates(None, user)
        if choices:
            self.fields["template"] = forms.ChoiceField(choices=choices)
        else:
            self.fields["template"] = forms.CharField()
        self.fields["title"].widget = forms.TextInput(attrs={'size': 30})

# class NewNewsletterForm(forms.ModelForm):
#     class Meta:
#         model = Newsletter
#         fields = ('subject', 'template', 'items')
#         #widgets = {}
#         #try:
#         #    widgets.update({
#         #        'items': ChosenSelectMultiple(),
#         #    })
#         #except NameError:
#         #    print 'NO ChosenSelectMultiple'
#         #    pass

#     def __init__(self, user, *args, **kwargs):
#         super(NewNewsletterForm, self).__init__(*args, **kwargs)
#         tpl_choices = get_newsletter_templates(None, user)
#         if tpl_choices:
#             self.fields["template"] = forms.ChoiceField(choices=tpl_choices)
#         else:
#             self.fields["template"] = forms.CharField()
#         self.fields["subject"].widget = forms.TextInput(attrs={'size': 30})


class PublishArticleForm(forms.ModelForm):
    class Meta:
        model = get_article_class()
        fields = ('publication',)# 'summary', 'category')
        widgets = {
            'publication': forms.HiddenInput(),
        }

    #def __init__(self, *args, **kwargs):
    #    article = kwargs['instance']
    #    try:
    #        initials = kwargs['initial']
    #    except:
    #        initials = {}
    #    summary = article.summary
    #    if not summary:
    #        summary = dehtml(article.content)[:250]
    #    initials.update({'summary': summary})
    #    kwargs['initial'] = initials
    #    super(PublishArticleForm, self).__init__(*args, **kwargs)


# class NewsletterForm(floppyforms.ModelForm):

#     class Meta:
#         model = Newsletter
#         fields = ('content',)
#         widgets = {
#             'content': AlohaInput(text_color_plugin=False),
#         }

#     class Media:
#         css = {
#             'all': ('css/colorbox.css',),
#         }
#         js = ('js/jquery.form.js', 'js/jquery.pageslide.js', 'js/jquery.colorbox-min.js', 'js/colorbox.coop.js')


# class NewsletterSchedulingForm(floppyforms.ModelForm):
#     class Meta:
#         model = NewsletterSending
#         fields = ('scheduling_dt',)

#     def clean_scheduling_dt(self):
#         sch_dt = self.cleaned_data['scheduling_dt']

#         if not sch_dt:
#             raise ValidationError(_(u"This field is required"))

#         if sch_dt < datetime.now():
#             raise ValidationError(_(u"The scheduling date must be in future"))

#         return sch_dt

# class NewsletterTemplateForm(forms.Form):

#     def __init__(self, newsletter, user, *args, **kwargs):
#         super(NewsletterTemplateForm, self).__init__(*args, **kwargs)
#         choices = get_newsletter_templates(newsletter, user)
#         if choices:
#             self.fields["template"] = forms.ChoiceField(choices=choices)
#         else:
#             self.fields["template"] = forms.CharField()
#         self.fields["template"].initial = newsletter.template

# class NewsletterAdminForm(forms.ModelForm):

#     def __init__(self, *args, **kwargs):
#         super(NewsletterAdminForm, self).__init__(*args, **kwargs)
#         self.newsletter = kwargs.get('instance', None)
#         choices = get_newsletter_templates(self.newsletter, self.current_user)
#         if choices:
#             self.fields["template"] = forms.ChoiceField(choices=choices)
#         else:
#             self.fields["template"] = forms.CharField()

#     class Meta:
#         model = Newsletter
#         fields = ('subject', 'content', 'template', 'items')
#         widgets = {}
#         try:
#             widgets.update({
#                 'items': ChosenSelectMultiple(),
#             })
#         except NameError:
#             print 'No ChosenSelectMultiple'
#             pass

#     class Media:
#         css = {
#             'all': ('css/admin-tricks.css',),
#         }
#         js = ()